package nz.ac.canterbury.debugassist;

import java.util.ArrayList;
import java.util.HashSet;
import nz.ac.canterbury.debugassist.datasource.DataEventStoreTime;
import nz.ac.canterbury.debugassist.events.*;
import nz.ac.canterbury.debugassist.handlers.EclipseDebuggerEventListenerManager;
import nz.ac.canterbury.debugassist.model.BreakpointEvent;
import nz.ac.canterbury.debugassist.model.Variable;

/**
 * @author  James
 */
public class DebugAssistLogic implements DataUpdateBreakpointClient, DataUpdateEventClient{

	private static DebugAssistLogic instance;
	private HashSet<Integer> seenVariables;
	private DebugAssistLogic() {
		//connect event listeners
		new EclipseDebuggerEventListenerManager();
		DataUpdateBreakpointListener.getInstance().attachElement(this);
		DataUpdateEventListener.getInstance().attachElement(this);
	}
		

	public static DebugAssistLogic getInstance() {
		if(instance == null) {
			instance = new DebugAssistLogic();
		}
		
		return instance;
	}

	
	@Override
	public void breakpointUpdateRequired() {
		// TODO Auto-generated method stub
		//System.out.println("new breakpoint added");
		
	}

	@Override
	public void updateModelRequired() {
		// TODO Auto-generated method stub
		//System.out.println("model change required");
		//System.out.println(configView);
		
		
	}

	public int getBreakpointCount() {
		return DataEventStoreTime.getAllBreakpointEvents().size();
	}
	public void deleteAllBreakpointData() {
		DataEventStoreTime.clearEvents();
	}
	
	public void cleanDebugEvents(ArrayList<BreakpointEvent> events, boolean hidePrimitives, int depth) {
		for(BreakpointEvent event : events) {
			seenVariables = new HashSet<Integer>();
			ArrayList<Variable> newV = new ArrayList<Variable>();
			for(Variable v : event.getVariables().clone()) {
				
				if(!hidePrimitives || !v.isLeaf()) {
					newV.add(v);
					subClearDebugEvents(v,hidePrimitives, depth,0);
				}
				
				
			}
			event.updateVariables(newV.toArray(new Variable[0]));
		}
	}
	
	private void subClearDebugEvents(Variable parent, boolean hidePrimitives, int depthLimit, int currentDepth) {
		try {
		if(seenVariables.contains(parent.getObjectID())) { return; }
		seenVariables.add(parent.getObjectID());
		if(hidePrimitives) {
			parent.removePrimitives();
		}
		if(currentDepth == depthLimit) {
			parent.removeAllChildren();
			return;
		}
		if(!parent.isLeaf()) {
			for(Variable v : parent.getAllChildren()) {
				subClearDebugEvents(v,hidePrimitives,depthLimit,currentDepth+1);
			}
		}
		} catch(Exception ex) {
			System.out.println(ex);
		}
		
	}
	
	
	
	
	
	
}
